/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import javax.swing.*;
import javax.swing.table.*;

/**
   MARSTableModel is the intermediate between the host models stored in
   MARSConfig and the table that is the core of the MARS display window.
   It also acts as the UI thread that drives the display of new host
   data as it is available.
*/

public class MARSTableModel extends AbstractTableModel {

    /** Compile-time debugging switch. Should be false. */
    public static final boolean DEBUG = false;
    
    // Class variables
    static MARSTableModel currentModel;

    // Instance variables
    String[] headers;

    // Constants
    final static int HOST_POS = 0;

    /**
       Create a new MARSTableModel
    
       @return a new MARSTableModel that will get data from the current
       configuration
    */

    public MARSTableModel () {
	int i, j;

	// Dynamically build the column headers array
	headers = new String[1+MARS.SVC_COUNT+MARS.SPOT_COUNT];
	headers[0] = "Host";        // hostname always in first column
	j = 1;                      // start inserting at the second column
	for (i = 0; i < MARS.SVC_COUNT; i++)
	    headers[j++] = MARS.SVC[i];
	for (i = 0; i < MARS.SPOT_COUNT; i++)
	    headers[j++] = MARS.SPOT[i];

	// Set the current table model
	currentModel = this;
    }
    
    /**
       Update the current MARSTableModel by telling it to fire a
       table data changed event
    */

    public static void updateCurrent() {
	currentModel.fireTableDataChanged();
    }

    /**
       Return the number of columns (always constant)

       @return the number of columns
    */
    public int getColumnCount() {
	return headers.length;
    }
   
    /**
       Return the number of rows (equal to the number of hosts we're watching)

       @return the number of rows
    */
    public int getRowCount() {
	return MARSConfig.getCurrent().getHostCount();
    }

    /**
       Return the name of the ith column

       @param col the column whose name to get
       @return the name of the specified column
    */

    public String getColumnName(int col) {
	return headers[col];
    }

    /**
       Return the value at a specified location in the table

       @return the value to be displayed at the specified location
    */

    public Object getValueAt(int row, int col) {
        String retval;

	// First, get the host we're talking to
	Host thisHost = MARSConfig.getCurrent().getHost(row);

	// Then select the information appropriate for the column
	// Hostname always first column
	if (col == HOST_POS)
	    retval = thisHost.getVisibleName();
	// handle service columns
	else if (col > HOST_POS && col < (MARS.SVC_COUNT+1))
	    retval = thisHost.getSvcStatus(col-1);
	// handle SPOTS columns
	else if (col > MARS.SVC_COUNT && 
		 col < MARS.SVC_COUNT + MARS.SPOT_COUNT +1)
	    retval = thisHost.getSpotValue(col-(MARS.SVC_COUNT+1));
	// handle errors
	else
	    throw new IllegalArgumentException("Bad cell ("+row+", "+col+") while displaying host: "+thisHost.toString());
	
	return retval;
    }
}	    


