/*
  Copyright 1999 Brian H. Trammell (brian@altara.org)
  Copyright 1999 Ben Parrott (benji@altara.org)
  
  This software (and its source code) are freely usable, modifiable, 
  and redistributable under the terms of the GNU General Public License, 
  available at http://www.gnu.org/copyleft/gpl.html, and in the ./COPYING 
  file in the MARS distribution.
*/

package org.altara.mars;

import java.net.*;
import java.io.*;
import java.util.*;

/**
   This class is the client end of the connection to the SPOTS server. It
   connects to the SPOTS server, recieves the load average and percent
   free disk space separated by whitespace, and stores the information for
   retrieval.
*/

// Revision 0.1 : 31 Jan 1999 bht - initial entry
//                            GO FALCONS!!
// Revision 0.3 : 07 Feb 1999 bht - fixed javadoc
// Revision 1.2.0 : 21 Mar 1999 bht - fixed deep cloning bug
// Revision 1.2.2 : 25 Mar 1999 bht - modified for ease of spot addition
// Revision 1.4.2 :  6 May 1999 bht - added support for free memory information
// Revision 1.4.4 : 11 May 1999 bht - fixed the Bowers bug
// Revision 1.4.6 : 16 May 1999 bht - added per-filesystem free space support

public class SPOTSQuery implements Serializable {
    
    /** Compile-time debugging switch. Should be false. */
    public static final boolean DEBUG = false;
    
    /** The TCP port SPOTS listens on. Should be 24210. */
    public final static int SPOTSPORT = 24210;

    // Instance variables
    Host host;
    String value[] = new String[MARS.SPOT_COUNT];
    Hashtable fsinfo;

    /** 
	Creates a SPOTSQuery for a specified host.

	@param host the Host to query SPOTS on.
	@return a new SPOTSQuery for the specified host.
    */

    public SPOTSQuery (Host host) {
	this.host = host;
	fsinfo = new Hashtable();
    }

    /**
       Connects to the SPOTS server on the specified host, retrieves the
       system information and stores it in our instance variables.
       
       @return true if the query succeeded, false otherwise.
       @exception UnknownHostException if it couldn't find the target host
       @exception IOException if there was an error reading from the socket
       @exception InterruptedException if the timeout wait was interrupted
    */

    public boolean getInfo() throws UnknownHostException, IOException, InterruptedException {
	
	Socket spotSock;
	InputStreamReader spotsIn;
	StringBuffer spotsInBuf;
	StringTokenizer spotsTok;
	String mountpoint;
	String freepct;
	int thisChar;

	// Get the hostname out of our Host
	String hostname = host.getDNSName();

	if (DEBUG) System.out.println("mars.SQ.gI(): starting probe");
	
	// Open the spots query socket
	spotSock = new Socket(hostname,SPOTSPORT);
	if (DEBUG) 
	    System.out.println ("mars.SQ.gI(): opened socket "+hostname);

	// Get the input stream and create a buffer to receive data.
	if (DEBUG) 
	    System.out.println("mars.SQ.gI(): preparing to receive...");
	spotsIn = new InputStreamReader(spotSock.getInputStream());
	spotsInBuf = new StringBuffer();
 
	// Read from the stream until it closes
	if (DEBUG) 
	    System.out.print("mars.SQ.gI(): reading into buffer: ");
	while ((thisChar = spotsIn.read()) != -1) {
	    spotsInBuf.append((char)thisChar);
	    if (DEBUG) System.out.print((char)thisChar);
	}
	if (DEBUG) System.out.println(" done.");
	
	// Close the socket
	spotSock.close();

	// Now parse what we got from the SPOTS server.
	if (DEBUG) 
	    System.out.println("mars.SQ.gI(): parsing results");
	spotsTok = new StringTokenizer(spotsInBuf.toString());
	value[MARS.SPOT_LOAD] = spotsTok.nextToken();
        value[MARS.SPOT_DISK] = spotsTok.nextToken();
	value[MARS.SPOT_MEM] = spotsTok.nextToken();
	// Got all the mandatory tokens. Now fetch filesystem information.
	fsinfo.clear();
	while (spotsTok.hasMoreTokens()) {
	    mountpoint = spotsTok.nextToken();
	    freepct = spotsTok.nextToken();
	    fsinfo.put(mountpoint, freepct);
	    if (DEBUG) 
		System.out.println("mars.SQ.gI(): added to fsinfo: "+
				   "("+mountpoint+" => "+freepct+")");
	}

	// Determine that we got good results
	if ((value[MARS.SPOT_LOAD].indexOf('.') <= 0) || 
	    (value[MARS.SPOT_DISK].indexOf('%') <= 0))
	    return false;                     // return false if not.
	else
	    return true;                      // otherwise we're ok.
    }

    /**
       Return the specified value from the last getInfo() query
       
       @param spotNum the parameter whose value to get
       @return the specified value as of the last getInfo()

    */
    
    String getSpotValue(int spotNum) {
	return value[spotNum];
    }
    
    /**
       Return the filesystem information hashtable
       
       @return a Hashtable mapping mountpoints to free space percentages
    */

    Hashtable getFSInfo() {
	return fsinfo;
    }

    /**
       Return a clone of this SPOTSQuery
    */
    public Object clone() {
	try {
	    SPOTSQuery cloneProbe = (SPOTSQuery)super.clone();
	    return cloneProbe;
	} catch (CloneNotSupportedException ex) {
	    throw new InternalError();
	}
    }

}








